import traceback
import logging
from .session import org_mode

from .tests import tokenization
from .tests import basic
from .tests import gac_100
from .tests import gac_extension

logging.getLogger().setLevel(logging.ERROR)

tests = (
    ("tokenization", tokenization),
    # ("basic", basic),
    # ("gac 100", gac_100),
    # ("gac+", gac_extension),
)


def gen_session_name():
    return "treeNLU-test-session.org"


def main():
    org_mode.create_global_session(gen_session_name())
    failed = False
    for test_name, test_module in tests:
        try:
            with org_mode.global_session().log(test_name):
                test_module.main()
            print(" \x1b[1;32m✓\x1b[0m {}".format(test_name))
        except AssertionError as ae:
            print(" \x1b[1;31m✗\x1b[0m {}{}".format(test_name,
                                                  ('\n [Assertion] {}'.format(ae.args[0])) if len(ae.args) > 0
                                                   else ''))
            traceback.print_exc()
            failed = True

        except Exception as e:
            print(" \x1b[1;7;31m!\x1b[0m {}\n [Exception] {}".format(test_name, e))
            failed = True
            traceback.print_exc()
            raise
    org_mode.global_session().close()

    if failed:
        exit(1)

if __name__ == '__main__':
    main()
